package com.persagy.constant;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.entity.PonitData;
import com.persagy.entity.ReturnAlarmDefine;
import com.persagy.init.InitReturnAlarmConfig;

import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021年12月14日 下午2:04:41
 */
@Slf4j
public class ExtraCommonConstant {

	public static final Object sync = new Object();
	
	public static final String SUCCESS = "success";
	public static final String CRITERIA = "criteria";
    public static final String IBMS_SCENE_CODE = "ibmsSceneCode";
    public static final String IBMS_CLASS_CODE = "ibmsClassCode";
	public static final String RWD_INSTANCE_OBJECT_QUERY = "/rwd/instance/object/query?projectId={}&groupCode={}";
	
	public static String getObjectQueryUrl() {
		return CommonConst.zktDMP + StrUtil.format(RWD_INSTANCE_OBJECT_QUERY, CommonConst.projectId, CommonConst.groupCode);
	}
	
	/**
	 * 将对应的设备封装为报警定义
	 * 
	 * @param ibmsSceneCode
	 * @param ibmsClassCode
	 * @param alarmDefines 控返信息点定义
	 * @param meter2DefineKey 
	 * @param datas	设备对象数据
	 * @date 2021年12月14日 下午2:20:15
	 */
	public static void getAlarmDefines(String ibmsSceneCode, String ibmsClassCode, JSONArray alarmDefines, JSONArray datas) {
		try {
			synchronized (ExtraCommonConstant.sync) {
				for (int i = 0; i < datas.size(); i++) {
					JSONObject dataObject = datas.getJSONObject(i);
					String id = dataObject.getString("id");
					// 根据控返信息点，形成对应的报警定义，并加入临时缓存map
					for (int j = 0; j < alarmDefines.size(); j++) {
						// 获取详细的报警定义信息
						JSONObject alarmDefine = alarmDefines.getJSONObject(j);
						String controlCode = alarmDefine.getString("controlCode");
						String returnCode = alarmDefine.getString("returnCode");
						String alarmCondition = alarmDefine.getString("alarmCondition");
						JSONObject alarmCodes = alarmDefine.getJSONObject("alarmCode");
						
						String controlMeter = dataObject.getString(controlCode);
						String returnMeter = dataObject.getString(returnCode);
						if (StrUtil.isAllNotBlank(controlMeter, returnMeter)) {
							ReturnAlarmDefine define = getReturnAlarmDefine(controlCode, controlMeter, returnCode, returnMeter, alarmCodes);
							define.setAlarmCondition(alarmCondition);
							define.setIbmsSceneCode(ibmsSceneCode);
							define.setIbmsClassCode(ibmsClassCode);
							define.setEquipId(id);
							
							InitReturnAlarmConfig.controlCode2AlarmDefine.put(controlMeter, define);
							InitReturnAlarmConfig.returnCode2AlarmDefine.put(returnMeter, define);
							
							// 默认不做任何处理
							String alarmKey = getKey(alarmCodes.getString("meter"), alarmCodes.getString("funcid"));
							InitReturnAlarmConfig.ibmsCode2AlarmResult.put(alarmKey, -1);
							InitReturnAlarmConfig.ibmsCode2AlarmData.put(alarmKey, alarmCodes);
						}
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			log.error("查询管理范围设备信息失败", e);
		}
	}
	
	/**
	 * 获取此设备下对应信息点的对应报警定义
	 * 
	 * @param controlCode
	 * @param controlMeter 表号-功能号
	 * @param returnCode
	 * @param returnMeter 表号-功能号
	 * @return
	 * @date 2021年12月14日 下午2:52:09
	 */
	private static ReturnAlarmDefine getReturnAlarmDefine(String controlCode, String controlMeter, String returnCode, String returnMeter, JSONObject alarmCodes) {
		ReturnAlarmDefine define = new ReturnAlarmDefine();
		define.setGroupCode(CommonConst.groupCode);
		define.setProjectId(CommonConst.projectId);
		define.setBuilding(CommonConst.building);
		
		// 控制信息点信息
		PonitData controlData = new PonitData();
		String[] control = controlMeter.split("-");
		controlData.setInfoCode(controlCode);
		controlData.setMeter(control[0]);
		controlData.setFuncid(Integer.valueOf(control[1]));
		define.setControlInfo(controlData);
		
		// 反馈信息点信息
		PonitData returnData = new PonitData();
		String[] returnM = returnMeter.split("-");
		returnData.setInfoCode(returnCode);
		returnData.setMeter(returnM[0]);
		returnData.setFuncid(Integer.valueOf(returnM[1]));
		define.setReturnInfo(returnData);
		
		// 报警信息点信息
		PonitData alarmData = new PonitData();
		alarmData.setInfoCode(alarmCodes.getString("code"));
		alarmData.setMeter(alarmCodes.getString("meter"));
		alarmData.setFuncid(alarmCodes.getInteger("funcid"));
		define.setAlarmInfo(alarmData);
		
		return define;
	}
	
	/**
	 * 查询对应设备类下的所有设备数据
	 * 
	 * @param ibmsSceneCode
	 * @param ibmsClassCode
	 * @return 设备数据
	 * @date 2021年12月14日 下午2:20:15
	 */
	public static JSONArray getObjectArray(String ibmsSceneCode, String ibmsClassCode) {
		try {
			JSONObject params = new JSONObject();
			JSONObject criteria = new JSONObject();
	        criteria.put(IBMS_SCENE_CODE, ibmsSceneCode);
	        criteria.put(IBMS_CLASS_CODE, ibmsClassCode);
	        params.put(CRITERIA, criteria);
			String post = HttpUtil.post(getObjectQueryUrl(), params.toJSONString(), 10 * 1000);
			if (StrUtil.isBlank(post)) {
				return null;
			}
			JSONObject result = JSONObject.parseObject(post);
			if (!SUCCESS.equals(result.getString("result"))) {
				return null;
			}
			JSONArray datas = result.getJSONArray("data");
			if (datas == null || datas.size() == 0) {
				return null;
			}
			
			return datas;
		} catch (Exception e) {
			e.printStackTrace();
			log.error("查询管理范围设备信息失败", e);
		}
		return null;
	}
	
	/**
	 * key 拼接
	 * 
	 * @param value1
	 * @param value2
	 * @return value1-value2
	 * @date 2021年12月14日 下午2:31:28
	 */
    public static String getKey(String value1, String value2) {
        return value1 + "-" + value2;
    }
    
}
